# -*- coding: utf-8 -*-
from eic_aichat_users.sessions.persistence.SessionsMemoryPersistence import SessionsMemoryPersistence
from test.sessions.persistence.SessionsPersistenceFixture import SessionsPersistenceFixture


class TestSessionsMemoryPersistence:
    persistence: SessionsMemoryPersistence
    fixture: SessionsPersistenceFixture

    def setup_method(self):
        self.persistence = SessionsMemoryPersistence()
        self.fixture = SessionsPersistenceFixture(self.persistence)

        self.persistence.open(None)
        self.persistence.clear(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_crud_operations(self):
        self.fixture.test_crud_operations()

    def test_get_with_filter(self):
        self.fixture.test_close_expired()



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional
from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_components.config import ConfigParams

from ..data.SettingsSectionV1 import SettingsSectionV1


class ISettingsPersistence(ABC):
    @abstractmethod
    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        pass

    @abstractmethod
    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[SettingsSectionV1]:
        pass

    @abstractmethod
    def set(self, context: Optional[IContext], item: SettingsSectionV1) -> SettingsSectionV1:
        pass

    @abstractmethod
    def modify(self, context: Optional[IContext], id: str,
               update_params: ConfigParams, increment_params: ConfigParams) -> SettingsSectionV1:
        pass



# -*- coding: utf-8 -*-
import json

import bottle
from pip_services4_commons.convert import TypeCode
from pip_services4_components.context import Context
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_data.validate import ObjectSchema
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_components.config import ConfigParams

from eic_aichat_users.settings.logic.ISettingsService import ISettingsService
from eic_aichat_users.settings.data.SettingsSectionV1Schema import SettingsSectionV1Schema


class SettingsOperations(RestOperations):
    def __init__(self):
        super().__init__()
        self._settings_service: ISettingsService = None
        self._dependency_resolver.put("settings-service", Descriptor('aichatusers-settings', 'service', '*', '*', '1.0'))

    def configure(self, config):
        super().configure(config)

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._settings_service = self._dependency_resolver.get_one_required('settings-service')

    def get_section_ids(self):
        context = Context.from_trace_id(self._get_trace_id())
        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        try:
            res = self._settings_service.get_section_ids(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_sections(self):
        context = Context.from_trace_id(self._get_trace_id())
        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        try:
            res = self._settings_service.get_sections(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_section_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._settings_service.get_section_by_id(context, id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def set_section(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json or {}
        id = data.get("id")
        parameters = ConfigParams.from_value(data.get("parameters") or {})
        try:
            res = self._settings_service.set_section(context, id, parameters)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def modify_section(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json or {}
        id = data.get("id")
        update_params = ConfigParams.from_value(data.get("update_params") or {})
        increment_params = ConfigParams.from_value(data.get("increment_params") or {})
        try:
            res = self._settings_service.modify_section(context, id, update_params, increment_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def register_routes(self, controller: RestController):
        controller.register_route('get', 'users/<user_id>/settings/ids', None, self.get_section_ids)

        controller.register_route('get', 'users/<user_id>/settings', None, self.get_sections)

        controller.register_route('get', 'users/<user_id>/settings/<id>', ObjectSchema(True)
                                  .with_optional_property("id", TypeCode.String),
                                  self.get_section_by_id)

        controller.register_route('post', 'users/<user_id>/settings', ObjectSchema(True)
                                  .with_required_property("body", SettingsSectionV1Schema()),
                                  self.set_section)

        controller.register_route('post', 'users/<user_id>/settings/modify', ObjectSchema(True)
                                  .with_required_property("id", TypeCode.String)
                                  .with_optional_property("update_params", TypeCode.Map)
                                  .with_optional_property("increment_params", TypeCode.Map),
                                  self.modify_section)



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional
from datetime import datetime

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_commons.data import AnyValueMap

from ..data import SessionV1


class ISessionsPersistence(ABC):

    @abstractmethod
    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        raise NotImplementedError()

    @abstractmethod
    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[SessionV1]:
        raise NotImplementedError()

    @abstractmethod
    def create(self, context: Optional[IContext], item: SessionV1) -> SessionV1:
        raise NotImplementedError()

    @abstractmethod
    def update(self, context: Optional[IContext], item: SessionV1) -> SessionV1:
        raise NotImplementedError()

    @abstractmethod
    def update_partially(self, context: Optional[IContext], id: str, data: AnyValueMap) -> SessionV1:
        raise NotImplementedError()

    @abstractmethod
    def delete_by_id(self, context: Optional[IContext], id: str) -> Optional[SessionV1]:
        raise NotImplementedError()

    @abstractmethod
    def close_expired(self, context: Optional[IContext], request_time: datetime):
        raise NotImplementedError()



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Optional, Any

from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, FilterParams, PagingParams
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from ..data import AccountV1
from .IAccountsPersistence import IAccountsPersistence


class AccountsMongoDbPersistence(IdentifiableMongoDbPersistence, IAccountsPersistence):

    def __init__(self):
        super().__init__('accounts')
        self._max_page_size = 1000

        # Ensure unique index on login
        # self._ensure_index({'login': 1}, {'unique': True})

    def __compose_filter(self, filter_params: FilterParams) -> Any:
        filter_params = filter_params or FilterParams()
        criteria = []

        search = filter_params.get_as_nullable_string('search')
        if search:
            search_regex = {'$regex': search, '$options': 'i'}
            criteria.append({'$or': [{'login': search_regex}, {'name': search_regex}]})

        id = filter_params.get_as_nullable_string('id')
        if id:
            criteria.append({'_id': id})

        ids = filter_params.get_as_object('ids')
        if isinstance(ids, str):
            ids = ids.split(',')
        if isinstance(ids, list):
            criteria.append({'_id': {'$in': ids}})

        name = filter_params.get_as_nullable_string('name')
        if name:
            criteria.append({'name': name})

        login = filter_params.get_as_nullable_string('login')
        if login:
            criteria.append({'login': login})

        active = filter_params.get_as_nullable_boolean('active')
        if active is not None:
            criteria.append({'active': active})

        from_create_time = filter_params.get_as_nullable_datetime('from_create_time')
        if from_create_time:
            criteria.append({'create_time': {'$gte': from_create_time}})

        to_create_time = filter_params.get_as_nullable_datetime('to_create_time')
        if to_create_time:
            criteria.append({'create_time': {'$lt': to_create_time}})

        deleted = filter_params.get_as_boolean_with_default('deleted', False)
        if not deleted:
            criteria.append({'$or': [{'deleted': False}, {'deleted': {'$exists': False}}]})

        return {'$and': criteria} if criteria else {}

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams, sort: Any = None, select: Any = None) -> DataPage:
        sort = sort or '-create_time'
        select = select or {'custom_dat': 0}
        return super().get_page_by_filter(context, self.__compose_filter(filter_params), paging, sort, select)

    def get_one_by_login(self, context: Optional[IContext], login: str) -> Optional[AccountV1]:
        criteria = {'login': login}
        item = self._collection.find_one(criteria)

        self._logger.trace(context, "Retrieved from %s with login = %s", self._collection.name, login)
        item = self._convert_to_public(item)

        return item

    def get_one_by_id_or_login(self, context: Optional[IContext], id_or_login: str) -> Optional[AccountV1]:
        criteria = {
            '$or': [
                {'_id': id_or_login},
                {'login': id_or_login}
            ]
        }

        item = self._collection.find_one(criteria)
        self._logger.trace(context, "Retrieved from %s by %s", self._collection.name, id_or_login)
        item = self._convert_to_public(item)

        return item

    def create(self, context: Optional[IContext], item: AccountV1) -> AccountV1:
        if item is None:
            return None

        item.active = getattr(item, 'active', True)
        item.create_time = getattr(item, 'create_time', None) or datetime.utcnow()

        return super().create(context, item)



# -*- coding: utf-8 -*-
from typing import Optional
from pip_services4_components.context import IContext
from pip_services4_components.refer import IReferences, IReferenceable, Descriptor
from pip_services4_components.config import ConfigParams
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data.SettingsSectionV1 import SettingsSectionV1
from ..persistence.ISettingsPersistence import ISettingsPersistence
from .ISettingsService import ISettingsService


class SettingsService(ISettingsService, IReferenceable):
    __persistence: ISettingsPersistence = None

    def set_references(self, references: IReferences):
        self.__persistence = references.get_one_required(
            Descriptor('aichatusers-settings', 'persistence', '*', '*', '1.0')
        )

    def get_section_ids(self, context: Optional[IContext], filter_params: FilterParams,
                        paging: PagingParams) -> DataPage[str]:
        page = self.__persistence.get_page_by_filter(context, filter_params, paging)
        ids = [item.id for item in page.data]
        return DataPage(ids, page.total)

    def get_sections(self, context: Optional[IContext], filter_params: FilterParams,
                     paging: PagingParams) -> DataPage:
        return self.__persistence.get_page_by_filter(context, filter_params, paging)

    def get_section_by_id(self, context: Optional[IContext], id: str) -> ConfigParams:
        item = self.__persistence.get_one_by_id(context, id)
        return item.parameters if item else ConfigParams()

    def set_section(self, context: Optional[IContext], id: str,
                    parameters: ConfigParams) -> ConfigParams:
        item = SettingsSectionV1(id=id, parameters=parameters)
        item = self.__persistence.set(context, item)
        return item.parameters

    def modify_section(self, context: Optional[IContext], id: str,
                       update_params: ConfigParams,
                       increment_params: ConfigParams) -> ConfigParams:
        item = self.__persistence.modify(context, id, update_params, increment_params)
        return item.parameters



# -*- coding: utf-8 -*-
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from .IPasswordsPersistence import IPasswordsPersistence


class PasswordsMongoDbPersistence(IdentifiableMongoDbPersistence, IPasswordsPersistence):
    
    def __init__(self):
        super().__init__('passwords')
        self._max_page_size = 1000


# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional, List

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data import PartyActivityV1


class IActivitiesService(ABC):
    @abstractmethod
    def get_activities(self, context: IContext, filter_params: FilterParams, paging_params: PagingParams) -> DataPage:
        pass

    @abstractmethod
    def log_activity(self, context: IContext, activity: PartyActivityV1) -> PartyActivityV1:
        pass

    @abstractmethod
    def batch_log_activities(self, context: IContext, activities: List[PartyActivityV1]) -> None:
        pass

    @abstractmethod
    def delete_activities_by_filter(self, context: IContext, filter_params: FilterParams) -> None:
        pass 


# -*- coding: utf-8 -*-

__all__ = ['AccountsOperations', 'ActivitiesOperations', 'SettingsOperations', 'PasswordsOperations', 'RolesOperations', 'SessionsOperations']

from .AccountsOperations import AccountsOperations
from .ActivitiesOperations import ActivitiesOperations
from .SettingsOperations import SettingsOperations
from .PasswordsOperations import PasswordsOperations
from .RolesOperations import RolesOperations
from .SessionsOperations import SessionsOperations



# -*- coding: utf-8 -*-
from typing import Optional, List

from pip_services4_commons.convert import TypeCode
from pip_services4_data.validate import ObjectSchema, FilterParamsSchema, PagingParamsSchema
from pip_services4_data.validate import ArraySchema
from pip_services4_rpc.commands import CommandSet, ICommand, Command

from ..data import PartyActivityV1Schema


class ActivitiesCommandSet(CommandSet):
    _controller = None

    def __init__(self, controller):
        super().__init__()
        self._controller = controller

        self.add_command(self._make_get_activities_command())
        self.add_command(self._make_log_activity_command())
        self.add_command(self._make_batch_log_activities_command())
        self.add_command(self._make_delete_activities_by_filter_command())

    def _make_get_activities_command(self) -> ICommand:
        return Command(
            'get_activities',
            ObjectSchema(True)
                .with_optional_property('filter', FilterParamsSchema())
                .with_optional_property('paging', PagingParamsSchema()),
            lambda context, args: self._controller.get_activities(
                context,
                args.get('filter'),
                args.get('paging')
            )
        )

    def _make_log_activity_command(self) -> ICommand:
        return Command(
            'log_activity',
            ObjectSchema(True)
                .with_required_property('activity', PartyActivityV1Schema()),
            lambda context, args: self._controller.log_activity(
                context,
                args.get('activity')
            )
        )

    def _make_batch_log_activities_command(self) -> ICommand:
        return Command(
            'batch_log_activities',
            ObjectSchema(True)
                .with_required_property('activities', ArraySchema(PartyActivityV1Schema())),
            lambda context, args: self._controller.batch_log_activities(
                context,
                args.get('activities')
            )
        )

    def _make_delete_activities_by_filter_command(self) -> ICommand:
        return Command(
            'delete_activities_by_filter',
            ObjectSchema(True)
                .with_optional_property('filter', FilterParamsSchema()),
            lambda context, args: self._controller.delete_activities_by_filter(
                context,
                args.get('filter')
            )
        ) 


# -*- coding: utf-8 -*-
import json

import bottle
from pip_services4_commons.convert import TypeCode
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_data.validate import ObjectSchema
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_components.context import Context

from eic_aichat_users.accounts.data import AccountV1, AccountV1Schema
from eic_aichat_users.accounts.logic.IAccountsService import IAccountsService


class AccountsOperations(RestOperations):
    def __init__(self):
        super().__init__()
        self._accounts_service: IAccountsService = None
        self._dependency_resolver.put("accounts-service", Descriptor('aichatusers-accounts', 'service', '*', '*', '1.0'))

    def configure(self, config):
        super().configure(config)

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._accounts_service = self._dependency_resolver.get_one_required('accounts-service')

    def get_accounts(self):
        context = Context.from_trace_id(self._get_trace_id())
        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        try:
            res = self._accounts_service.get_accounts(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_account_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.get_account_by_id(context, id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_account_by_login(self, login):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.get_account_by_login(context, login)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_account_by_id_or_login(self, id_or_login):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.get_account_by_id_or_login(context, id_or_login)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def create_account(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json
        account = data if isinstance(data, dict) else json.loads(data)
        account = None if not account else AccountV1(**account)
        try:
            res = self._accounts_service.create_account(context, account)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def update_account(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json
        account = data if isinstance(data, dict) else json.loads(data)
        account = None if not account else AccountV1(**account)
        try:
            res = self._accounts_service.update_account(context, account)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def delete_account_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.delete_account_by_id(context, id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def drop_account_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._accounts_service.drop_account_by_id(context, id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def register_routes(self, controller: RestController):
        controller.register_route('get', '/users', None, self.get_accounts)

        controller.register_route('get', '/users/<id>', ObjectSchema(True)
                                  .with_optional_property("id", TypeCode.String),
                                  self.get_account_by_id)

        controller.register_route('get', '/accounts/by_login/<login>', ObjectSchema(True)
                                  .with_optional_property("login", TypeCode.String),
                                  self.get_account_by_login)

        controller.register_route('get', '/accounts/by_id_or_login/<id_or_login>', ObjectSchema(True)
                                  .with_optional_property("id_or_login", TypeCode.String),
                                  self.get_account_by_id_or_login)

        controller.register_route('post', '/users', ObjectSchema(True)
                                  .with_required_property("body", AccountV1Schema()),
                                  self.create_account)

        controller.register_route('put', '/users', ObjectSchema(True)
                                  .with_required_property("body", AccountV1Schema()),
                                  self.update_account)

        controller.register_route('delete', '/users/<id>', ObjectSchema(True)
                                  .with_required_property("id", TypeCode.String),
                                  self.delete_account_by_id)

        controller.register_route('delete', '/users/<id>/drop', ObjectSchema(True)
                                  .with_required_property("id", TypeCode.String),
                                  self.drop_account_by_id)



# -*- coding: utf-8 -*-
from datetime import datetime
from pip_services4_data.query import FilterParams, PagingParams
from pip_services4_commons.data import AnyValueMap

from eic_aichat_users.sessions.data.SessionV1 import SessionV1
from eic_aichat_users.sessions.persistence.ISessionsPersistence import ISessionsPersistence


SESSION1 = SessionV1(user_id='1', user_name='User 1')
SESSION2 = SessionV1(user_id='2', user_name='User 2')


class SessionsPersistenceFixture:
    def __init__(self, persistence: ISessionsPersistence):
        assert persistence is not None
        self._persistence = persistence

    def test_crud_operations(self):
        # Create the first session
        session1 = self._persistence.create(None, SESSION1)
        assert session1 is not None
        assert session1.id is not None
        assert session1.open_time is not None
        assert session1.request_time is not None
        assert session1.user_id == SESSION1.user_id

        # Create the second session
        session2 = self._persistence.create(None, SESSION2)
        assert session2 is not None
        assert session2.id is not None
        assert session2.open_time is not None
        assert session2.request_time is not None
        assert session2.user_id == SESSION2.user_id

        # Partially update session
        updated = self._persistence.update_partially(
            None,
            session1.id,
            AnyValueMap.from_tuples("data", "123")
        )
        assert updated is not None
        assert updated.id == session1.id
        assert updated.data == "123"

        # Get session by filter
        page = self._persistence.get_page_by_filter(
            None,
            FilterParams.from_tuples("user_id", "1"),
            PagingParams()
        )
        assert page is not None
        assert len(page.data) == 1

    def test_close_expired(self):
        self._persistence.close_expired(None, datetime.utcnow())



# -*- coding: utf-8 -*-
from eic_aichat_users.settings.persistence.SettingsMemoryPersistence import SettingsMemoryPersistence
from test.settings.persistence.SettingsPersistenceFixture import SettingsPersistenceFixture


class TestSettingsMemoryPersistence:
    persistence: SettingsMemoryPersistence
    fixture: SettingsPersistenceFixture

    def setup_method(self):
        self.persistence = SettingsMemoryPersistence()
        self.fixture = SettingsPersistenceFixture(self.persistence)
        self.persistence.open(None)
        self.persistence.clear(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_get_and_set(self):
        self.fixture.test_get_and_set()

    def test_set_parameter(self):
        self.fixture.test_set_parameter()

    def test_increment_parameter(self):
        self.fixture.test_increment_parameter()

    def test_get_sections(self):
        self.fixture.test_get_sections()



# -*- coding: utf-8 -*-
from typing import List, Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_persistence.persistence import IdentifiableMemoryPersistence

from ..data import PartyActivityV1


class ActivitiesMemoryPersistence(IdentifiableMemoryPersistence):
    """
    Memory persistence for party activities.
    """
    
    def __init__(self):
        super().__init__()
        self._max_page_size = 1000

    def __compose_filter(self, filter_params: FilterParams):
        filter_params = filter_params or FilterParams()
        
        id = filter_params.get_as_nullable_string('id')
        party_id = filter_params.get_as_nullable_string('party_id')
        ref_item_id = filter_params.get_as_nullable_string('ref_item_id')
        ref_parent_id = filter_params.get_as_nullable_string('ref_parent_id')
        ref_party_id = filter_params.get_as_nullable_string('ref_party_id')
        type = filter_params.get_as_nullable_string('type')
        
        def filter_activities(item: PartyActivityV1) -> bool:
            if id is not None and item.id != id:
                return False
                
            if party_id is not None and (item.party is None or item.party.id != party_id):
                return False
                
            if ref_item_id is not None and (item.ref_item is None or item.ref_item.id != ref_item_id):
                return False
                
            if ref_parent_id is not None:
                if item.ref_parents is None or len(item.ref_parents) == 0:
                    return False
                    
                found = False
                for ref_parent in item.ref_parents:
                    if ref_parent.id == ref_parent_id:
                        found = True
                        break
                        
                if not found:
                    return False
                    
            if ref_party_id is not None and (item.ref_party is None or item.ref_party.id != ref_party_id):
                return False
                
            if type is not None and item.type != type:
                return False
                
            return True
            
        return filter_activities

    def get_page_by_filter(self, context: IContext, filter_params: FilterParams, paging_params: PagingParams) -> DataPage:
        """
        Gets a page of party activities by specified filter
        
        Args:
            context: Operation context
            filter_params: Filter parameters
            paging_params: Paging parameters
            
        Returns:
            DataPage with retrieved party activities
        """
        return super().get_page_by_filter(context, self.__compose_filter(filter_params), paging_params)

    def delete_by_filter(self, context: IContext, filter_params: FilterParams) -> None:
        """
        Deletes party activities by filter
        
        Args:
            context: Operation context
            filter_params: Filter parameters
        """
        filter_function = self.__compose_filter(filter_params)
        
        items_to_remove = []
        for item in self._items:
            if filter_function(item):
                items_to_remove.append(item)
                
        for item in items_to_remove:
            self._items.remove(item) 


# -*- coding: utf-8 -*-

from pip_services4_observability.log import ILogger
from ...activities.data import PartyActivityV1, ReferenceV1
from ...activities.logic import IActivitiesService
from ..data.version1.PasswordActivityTypeV1 import PasswordActivityTypeV1


class ActivitiesConnector:
    def __init__(self, logger: ILogger, activities_service: IActivitiesService):
        self._logger = logger
        self._activities_service = activities_service

        if self._activities_service is None:
            self._logger.warn(None, 'Activities service was not found. Logging password activities is disabled')

    def log_activity(self, correlation_id: str, user_id: str, activity_type: str):
        if self._activities_service is None:
            return

        party = ReferenceV1(user_id, 'account', None)
        activity = PartyActivityV1(None, None, activity_type, party)

        try:
            self._activities_service.log_activity(correlation_id, activity)
        except Exception as err:
            self._logger.error(correlation_id, err, 'Failed to log user activity')

    def log_signin_activity(self, correlation_id: str, user_id: str):
        self.log_activity(correlation_id, user_id, PasswordActivityTypeV1.Signin)

    def log_password_recovered_activity(self, correlation_id: str, user_id: str):
        self.log_activity(correlation_id, user_id, PasswordActivityTypeV1.PasswordRecovered)

    def log_password_changed_activity(self, correlation_id: str, user_id: str):
        self.log_activity(correlation_id, user_id, PasswordActivityTypeV1.PasswordChanged)



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Optional, Callable, Any
from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, FilterParams, PagingParams, SortParams
from pip_services4_persistence.persistence import IdentifiableMemoryPersistence
from pip_services4_commons.errors import BadRequestException

from ..data import AccountV1
from .IAccountsPersistence import IAccountsPersistence


class AccountsMemoryPersistence(IdentifiableMemoryPersistence, IAccountsPersistence):

    def __init__(self):
        super().__init__()

    def __match_string(self, value: str, search: str) -> bool:
        if value is None and search is None:
            return True
        if value is None or search is None:
            return False
        return search.lower() in value.lower()

    def __match_search(self, item: AccountV1, search: str) -> bool:
        return self.__match_string(item.name, search) or self.__match_string(item.login, search)

    def __compose_filter(self, filter: FilterParams) -> Callable[[AccountV1], bool]:
        filter = filter or FilterParams()

        search = filter.get_as_nullable_string('search')
        id = filter.get_as_nullable_string('id')
        ids = filter.get_as_object('ids')
        name = filter.get_as_nullable_string('name')
        login = filter.get_as_nullable_string('login')
        active = filter.get_as_nullable_boolean('active')
        from_create_time = filter.get_as_nullable_datetime('from_create_time')
        to_create_time = filter.get_as_nullable_datetime('to_create_time')
        deleted = filter.get_as_boolean_with_default('deleted', False)

        if isinstance(ids, str):
            ids = ids.split(',')
        if not isinstance(ids, list):
            ids = None

        def filter_func(item: AccountV1) -> bool:
            if id and item.id != id:
                return False
            if ids and item.id not in ids:
                return False
            if name and item.name != name:
                return False
            if login and item.login != login:
                return False
            if active is not None and item.active != active:
                return False
            if from_create_time and item.create_time < from_create_time:
                return False
            if to_create_time and item.create_time >= to_create_time:
                return False
            if not deleted and getattr(item, 'deleted', False):
                return False
            if search and not self.__match_search(item, search):
                return False
            return True

        return filter_func

    def get_page_by_filter(self, context: Optional[IContext], filter: FilterParams,
                           paging: PagingParams, sort: SortParams = None, select: Any = None) -> DataPage:
        return super().get_page_by_filter(context, self.__compose_filter(filter), paging)

    def get_one_by_login(self, context: Optional[IContext], login: str) -> Optional[AccountV1]:
        item = next((x for x in self._items if x.login == login), None)
        return item

    def get_one_by_id_or_login(self, context: Optional[IContext], id_or_login: str) -> Optional[AccountV1]:
        item = next((x for x in self._items if x.id == id_or_login or x.login == id_or_login), None)
        return item

    def create(self, context: Optional[IContext], item: AccountV1) -> AccountV1:
        if item is None:
            return None

        existing = next((x for x in self._items if x.login == item.login), None)
        if existing:
            raise BadRequestException(context, 'ALREADY_EXIST', f"Account {item.login} already exists")\
                .with_details('login', item.login)

        item.active = getattr(item, 'active', True)
        item.create_time = getattr(item, 'create_time', None) or datetime.now()

        return super().create(context, item)



"""
AI Chat Users in Python
----------------------

Links
`````

* `development version <https://bitbucket.org/entinco/eic-ai-prototypes/src/main/lib-aichatusers-python>`

"""

from setuptools import find_packages
from setuptools import setup

try:
    readme = open('readme.md').read()
except:
    readme = __doc__

setup(
    name='eic_aichat_users',
    version='1.0.10',
    url='https://bitbucket.org/entinco/eic-ai-prototypes/src/master/lib-aichatusers-python',
    license='Commercial',
    author='Enterprise Innovation Consulting LLC',
    author_email='seroukhov@entinco.com',
    description='AI Chat Users in Python',
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=['config', 'data', 'test']),
    include_package_data=True,
    zip_safe=True,
    platforms='any',
    install_requires=[
        'pip-services4-commons>=0.0.0',
        'pip-services4-components>=0.0.0',
        'pip-services4-config>=0.0.0',
        'pip-services4-data>=0.0.0',
        'pip-services4-http>=0.0.0',
        'pip-services4-mongodb>=0.0.0',
        'pip-services4-persistence>=0.0.0',
        'pip-services4-prometheus>=0.0.0',
        'pip-services4-rpc>=0.0.0',
        'pip-services4-swagger>=0.0.0'
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: Other/Proprietary License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.12',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



# -*- coding: utf-8 -*-
from datetime import datetime
from pip_services4_data.query import FilterParams, PagingParams
from eic_aichat_users.passwords.data.version1.UserPasswordV1 import UserPasswordV1
from eic_aichat_users.passwords.persistence.IPasswordsPersistence import IPasswordsPersistence


USER_PWD = UserPasswordV1('1', 'password123')


class PasswordsPersistenceFixture:
    _persistence: IPasswordsPersistence

    def __init__(self, persistence: IPasswordsPersistence):
        assert persistence is not None
        self._persistence = persistence

    def test_crud_operations(self):
        # Create user password
        user_password = self._persistence.create(None, USER_PWD)

        self._assert_is_object(user_password)
        assert user_password.id == USER_PWD.id
        assert user_password.password is not None
        assert user_password.locked is False

        # Update the user password
        user_password = UserPasswordV1(USER_PWD.id, 'newpwd123')
        user_password.rec_code = "123"
        user_password.rec_expire_time = datetime.utcnow()

        user_password = self._persistence.update(None, user_password)

        self._assert_is_object(user_password)
        assert user_password.id == USER_PWD.id
        assert user_password.password == 'newpwd123'

        # Get user password
        user_password = self._persistence.get_one_by_id(None, USER_PWD.id)

        self._assert_is_object(user_password)
        assert user_password.id == USER_PWD.id

        # Delete the user password
        self._persistence.delete_by_id(None, USER_PWD.id)

        # Try to get deleted user
        user_password = self._persistence.get_one_by_id(None, USER_PWD.id)
        assert user_password is None

    def _assert_is_object(self, obj):
        assert obj is not None and isinstance(obj, object)




# -*- coding: utf-8 -*-

__all__ = ['IActivitiesService', 'ActivitiesService', 'ActivitiesCommandSet']

from .IActivitiesService import IActivitiesService
from .ActivitiesService import ActivitiesService
from .ActivitiesCommandSet import ActivitiesCommandSet 


# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Optional
from pymongo import ReturnDocument

from pip_services4_components.context import IContext
from pip_services4_components.config import ConfigParams
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from ..data.SettingsSectionV1 import SettingsSectionV1
from .ISettingsPersistence import ISettingsPersistence


class SettingsMongoDbPersistence(IdentifiableMongoDbPersistence, ISettingsPersistence):

    def __init__(self):
        super().__init__('settings')

    def _map_from_public(self, params: dict) -> dict:
        result = {}
        for key, value in params.items():
            new_key = key.replace('.', '_dot_')
            result[new_key] = value
        return result

    def _map_to_public(self, params: dict) -> dict:
        result = {}
        for key, value in params.items():
            new_key = key.replace('_dot_', '.')
            result[new_key] = value
        return result

    def convert_to_public(self, value: any) -> any:
        if value is None:
            return None
        return SettingsSectionV1(
            id=value.get('_id'),
            parameters=ConfigParams.from_value(self._map_to_public(value.get('parameters', {})))
        )

    def convert_from_public(self, item: SettingsSectionV1) -> any:
        return {
            '_id': item.id,
            'parameters': self._map_from_public(item.parameters.get_as_object()),
            'update_time': item.update_time
        }

    def _compose_filter(self, filter_params: FilterParams) -> dict:
        filter_params = filter_params or FilterParams()
        criteria = []

        if search := filter_params.get_as_nullable_string('search'):
            regex = {'$regex': search, '$options': 'i'}
            criteria.append({'_id': regex})

        if id := filter_params.get_as_nullable_string('id'):
            criteria.append({'_id': id})

        if id_starts := filter_params.get_as_nullable_string('id_starts'):
            regex = {'$regex': f'^{id_starts}', '$options': 'i'}
            criteria.append({'_id': regex})

        return {'$and': criteria} if criteria else {}

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        return super().get_page_by_filter(context, self._compose_filter(filter_params), paging, None, None)

    def set(self, context: Optional[IContext], item: SettingsSectionV1) -> SettingsSectionV1:
        item.update_time = datetime.now()
        update = {
            '$set': {
                'parameters': self._map_from_public(item.parameters.get_as_object()),
                'update_time': item.update_time
            }
        }

        result = self._collection.find_one_and_update(
            {'_id': item.id}, update,
            return_document=ReturnDocument.AFTER,
            upsert=True
        )

        self._logger.trace(context, "Set settings with id=%s", item.id)
        return self.convert_to_public(result)

    def modify(self, context: Optional[IContext], id: str,
               update_params: ConfigParams, increment_params: ConfigParams) -> SettingsSectionV1:
        update = {'$set': {'update_time': datetime.now()}}

        if update_params:
            for key, value in update_params.items():
                mongo_key = 'parameters.' + key.replace('.', '_dot_')
                update['$set'][mongo_key] = value

        if increment_params:
            update['$inc'] = {}
            for key, inc in increment_params.items():
                mongo_key = 'parameters.' + key.replace('.', '_dot_')
                update['$inc'][mongo_key] = int(inc)

        result = self._collection.find_one_and_update(
            {'_id': id},
            update,
            return_document=ReturnDocument.AFTER,
            upsert=True
        )

        self._logger.trace(context, "Modified settings with id=%s", id)
        return self.convert_to_public(result)
    
    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[SettingsSectionV1]:
        item = self._collection.find_one({'_id': id})
        if item:
            self._logger.trace(context, "Nothing found from %s with id = %s", self._collection_name, id)
        else:
            self._logger.trace(context, "Retrieved from %s with id = %s", self._collection_name, id)

        item = self.convert_to_public(item)
        return item



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Dict, List, Any, Optional

from pip_services4_data.data import IStringIdentifiable
from pip_services4_commons.data import StringValueMap
from .ReferenceV1 import ReferenceV1


class PartyActivityV1(IStringIdentifiable):
    """
    Data object that represents a party activity.
    
    Attributes:
        id (str): Unique activity id
        time (datetime): Time when activity took place
        type (str): Activity type
        party (ReferenceV1): Reference to party who generated the activity
        ref_item (ReferenceV1): Reference to item that was subject of activity
        ref_parents (List[ReferenceV1]): References to parent items
        ref_party (ReferenceV1): Reference to party who was subject of activity
        details StringValueMap: Map with additional details
    """
    
    def __init__(self, id: str = None, time: datetime = None, type: str = None, 
                 party: ReferenceV1 = None, ref_item: ReferenceV1 = None, 
                 ref_parents: List[ReferenceV1] = None, ref_party: ReferenceV1 = None, 
                 details: StringValueMap = None):
        """
        Creates a new party activity
        
        Args:
            id: Unique activity id
            time: Time when activity took place
            type: Activity type
            party: Reference to party who generated the activity
            ref_item: Reference to item that was subject of activity
            ref_parents: References to parent items
            ref_party: Reference to party who was subject of activity
            details: Map with additional details
        """
        self.id = id
        self.time = time or datetime.now()
        self.type = type
        self.party = party
        self.ref_item = ref_item
        self.ref_parents = ref_parents or []
        self.ref_party = ref_party
        self.details = details or {}


# -*- coding: utf-8 -*-
from eic_aichat_users.passwords.persistence.PasswordsMemoryPersistence import PasswordsMemoryPersistence
from test.passwords.persistence.PasswordsPersistenceFixture import PasswordsPersistenceFixture


class TestPasswordsMemoryPersistence:
    persistence: PasswordsMemoryPersistence
    fixture: PasswordsPersistenceFixture

    def setup_method(self):
        self.persistence = PasswordsMemoryPersistence()
        self.fixture = PasswordsPersistenceFixture(self.persistence)

        self.persistence.open(None)
        self.persistence.clear(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_crud_operations(self):
        self.fixture.test_crud_operations()



# -*- coding: utf-8 -*-
from eic_aichat_users.activities.persistence import ActivitiesMemoryPersistence
from test.activities.persistence.ActivitiesPersistenceFixture import ActivitiesPersistenceFixture


class TestActivitiesMemoryPersistence:
    persistence = None
    fixture = None

    @classmethod
    def setup_class(cls):
        cls.persistence = ActivitiesMemoryPersistence()
        cls.fixture = ActivitiesPersistenceFixture(cls.persistence)

    def setup_method(self, method):
        self.persistence.clear(None)

    def test_crud_operations(self):
        self.fixture.test_crud_operations()

    def test_get_with_filter(self):
        self.fixture.test_get_with_filter() 